import os
from collections import namedtuple

from sakee.colors import Colors
from sakee.stub import KodiStub


# Custom AddonData type
AddonData = namedtuple('AddonData', [
    'kodi_home_path',  # data path (either portable of user path)
    'add_on_id',  # the add-on id
    'add_on_path',  # the full path to the add-on
    'kodi_profile_path'  # the full path to the add-on profile folder
])

__add_on_infos = {}


def get_add_on_info_from_calling_script(add_on_id=None, print_info=False):
    if add_on_id is not None:
        # Always print details for specific add-ons
        print_info = True

    add_on_info = __add_on_infos.get(add_on_id)
    if add_on_info:
        return add_on_info

    # What is the Kodi Home path?
    assert os.path.isfile("addon.xml"), "Working directory outside add-on path: {}".format(os.getcwd())
    calling_add_on_path = os.getcwd()

    # Get the generic Kodi paths
    kodi_home_path = os.environ.get("KODI_HOME")
    if kodi_home_path:
        # We only need the ID from the path.
        _, path_add_on_id = calling_add_on_path.rsplit(os.sep, 1)
    else:
        # determine it based on the running path
        kodi_home_path, addon_name, path_add_on_id = os.getcwd().rsplit(os.sep, 2)

    kodi_home_path = os.path.abspath(kodi_home_path)
    assert os.path.isdir(kodi_home_path), \
        "Kodi home path (special://home) does not exist: '{}'".format(kodi_home_path)

    # Find the very first script called to determine the add-on ID if it was not specified
    add_on_id = add_on_id or path_add_on_id

    # Active add-on path
    if add_on_id is None or add_on_id == path_add_on_id:
        # We should use the data from the current calling add-on
        add_on_path = calling_add_on_path
    else:
        # We should set it to the requested add-on based on the given add-on ID
        add_on_path = os.path.join(kodi_home_path, "addons", add_on_id)
        if not os.path.isdir(add_on_path) and "portable_data" in calling_add_on_path:
            add_on_path = os.path.abspath(os.path.join(kodi_home_path, "..", "addons", add_on_id))
    assert os.path.isdir(add_on_path), \
        "Invalid add-on dir for add-on '{}': {}".format(add_on_id, add_on_path)

    # The active profile path
    if "KODI_PROFILE" in os.environ:
        add_on_profile_path = os.path.abspath(os.environ["KODI_PROFILE"])
    else:
        add_on_profile_path = os.path.join(kodi_home_path, "userdata")

    add_on_profile_path = os.path.abspath(add_on_profile_path)
    assert os.path.isdir(add_on_profile_path), \
        "Invalid Kodi master profile dir (special://masterprofile): {}".format(add_on_profile_path)

    if "KODI_ACTIVE_PROFILE" in os.environ:
        add_on_profile_path = os.path.join(
            add_on_profile_path, "profiles", os.environ["KODI_ACTIVE_PROFILE"])

        assert os.path.isdir(add_on_profile_path), \
            "Invalid Kodi profile dir (special://profile): {}".format(add_on_profile_path)

    a = AddonData(
        kodi_home_path=kodi_home_path,
        add_on_id=add_on_id,
        add_on_path=add_on_path,
        kodi_profile_path=add_on_profile_path
    )

    __add_on_infos[add_on_id] = a
    if add_on_id == path_add_on_id:
        __add_on_infos[None] = a

    if not print_info:
        return a

    KodiStub.print_line(
        "Found Add-on info: \n"
        "- Kodi Home (special://home):       {} \n"
        "- Add-on ID:                        {} \n"
        "- Add-on Path:                      {} \n"
        "- Kodi Profile (special://profile): {} \n"
            .format(a.kodi_home_path, a.add_on_id, a.add_on_path, a.kodi_profile_path),
        color=Colors.Blue
    )
    return a



# SPDX-License-Identifier: GPL-3.0

import io
import os
import re

from sakee import addoninfo


class JsonRpcApi(object):
    __ADDON_INFO = None

    def __init__(self):
        """ Initialise the JSON RPC API Implementation. """
        if JsonRpcApi.__ADDON_INFO is None:
            JsonRpcApi.__ADDON_INFO = addoninfo.get_add_on_info_from_calling_script()

    def handle(self, json_data):
        """ Handle the JSON RPC Request

        :param obj json_data: The JSON RPC request.

        :return: The JSON RPC reply.
        :rtype: dict
        """
        (class_name, method_name) = json_data["method"].split('.')

        # Find class
        try:
            class_reference = getattr(JsonRpcApi, class_name)
        except AttributeError:
            raise NotImplementedError

        # Find method
        class_instance = class_reference(JsonRpcApi.__ADDON_INFO)
        try:
            method_reference = getattr(class_instance, method_name)
        except AttributeError:
            raise NotImplementedError

        # Invoke method
        result = method_reference(**json_data["params"])

        return dict(
            id=0,
            jsonrpc='2.0',
            result=result,
        )

    class Settings(object):
        """ Allows manipulation of Kodi settings. """
        __SETTINGS = None

        def __init__(self, addon_info):
            """ Initialise the JSON RPC API Settings Namespace.

            :param obj addon_info:   Information about the current Add-on paths.
            """
            self._ADDON_INFO = addon_info

            # Load the guisettings.xml file
            if JsonRpcApi.Settings.__SETTINGS is None:
                settings_xml_file = os.path.join(self._ADDON_INFO.kodi_profile_path, "guisettings.xml")
                if os.path.isfile(settings_xml_file):
                    with io.open(settings_xml_file, encoding='utf-8') as fp:
                        settings_xml = fp.read()
                    results = re.findall(r'id="([^"]+)"[^>]*>([^<]+)<', settings_xml)
                    JsonRpcApi.Settings.__SETTINGS = dict(results)

        # noinspection PyPep8Naming
        def GetSettingValue(self, setting):  # NOSONAR
            """ Retrieves the value of a setting.

            :param str setting:  The name of the settings for which the value is retrieved.

            :return: The value for the given setting.
            :rtype: str
            """
            value = JsonRpcApi.Settings.__SETTINGS.get(setting, '')
            if value == 'true':
                value = True

            return dict(
                value=value
            )



# SPDX-License-Identifier: GPL-3.0

from sakee.colors import Colors
from sakee.stub import KodiStub

NOTIFICATION_INFO = "info"
NOTIFICATION_WARNING = "warning"
NOTIFICATION_ERROR = "error"


class Dialog(KodiStub):
    def __init__(self):
        super(Dialog, self).__init__()

    def ok(self, heading, message):  # NOSONAR
        """ OK dialog

        The functions permit the call of a dialog of information, a confirmation
        of the user by press from OK required.

        :param str heading:      dialog heading.
        :param str message:      dialog message.

        :return: Returns True if 'Ok' was pressed, else False.
        :rtype: bool

        Example::

            dialog = xbmcgui.Dialog()
            ok = dialog.ok('Kodi', 'There was an error.')

        """

        KodiStub.print_heading(heading)

        if KodiStub.is_interactive:
            self.read_input("{}. OK?".format(message), color=Colors.Yellow)
        else:
            KodiStub.print_line("{}. OK?".format(message), Colors.Yellow)
        return True

    # noinspection PyUnusedLocal
    def textviewer(self, heading, text, usemono=False):
        """ The text viewer dialog can be used to display descriptions, help texts or other larger texts.

        :param str heading:     Dialog heading.
        :param str text:        Dialog text.
        :param bool usemono:    Use a monospace font.

        """

        text = self.replace_colors(text)
        text = text.replace("\\n", "\n")
        self.print_heading(heading)
        self.print_line(text)
        self.print_line("=" * 120, color=Colors.Yellow)

        if KodiStub.is_interactive:
            self.read_input("OK?", color=Colors.Yellow)
        else:
            KodiStub.print_line("OK?", Colors.Yellow)
        return True

    # noinspection PyPep8Naming,PyUnusedLocal
    def multiselect(self, heading, options, autoclose=0, preselect=None, useDetails=False):  # NOSONAR
        """ Show a multi-select dialog.

        :param str heading:                     Dialog heading.
        :param list[string|ListItem] options:   Otions to choose from.
        :param int autoclose:                   Milliseconds to autoclose dialog. (default=do not autoclose)
        :param list[int]|None preselect:        Indexes of items to preselect in list (default: do not preselect any item)
        :param bool useDetails:                 Use detailed list instead of a compact list. (default=false)

        :return: Returns the selected items as a list of indices, or None if cancelled.
        :rtype: list[int]

        """

        self.print_heading(heading)

        selections = []
        for i in range(0, len(options)):
            self.print_line("{} ) {}".format(i, options[i]))
            selections.append(str(i))
        self.print_line("=" * 120, color=Colors.Yellow)
        selections = self.read_input("What items to select (%s)? " % (",".join(selections)),
                                     color=Colors.Yellow).lower()
        if not selections:
            return None
        return list(map(lambda index_value: int(index_value), selections.split(",")))

    # noinspection PyPep8Naming,PyUnusedLocal
    def select(self, heading, options, autoclose=0, preselect=None, useDetails=False):  # NOSONAR
        """ Show a select dialog.

        :param str heading:                     Dialog heading.
        :param list[string|ListItem] options:   Otions to choose from.
        :param int autoclose:                   Milliseconds to autoclose dialog. (default=do not autoclose)
        :param int|None preselect:              Indexes of items to preselect in list (default: do not preselect any item)
        :param bool useDetails:                 Use detailed list instead of a compact list. (default=false)

        :return: Returns the selected items as a list of indices, or None if cancelled.
        :rtype: int

        """
        self.print_heading(heading)

        selections = []
        for i in range(0, len(options)):
            self.print_line("{} ) {}".format(i, options[i]))
            selections.append(str(i))
        self.print_line("=" * 120, color=Colors.Yellow)
        selections = self.read_input("What item to select (%s)? " % (",".join(selections)), color=Colors.Yellow).lower()
        if not selections:
            return None
        return list(map(lambda index_value: int(index_value), selections.split(",")))[0]

    # noinspection PyUnusedLocal
    def yesno(self, heading, message, nolabel='', yeslabel='', customlabel=None, autoclose=0):
        """ The Yes / No dialog can be used to inform the user about questions and get the answer.

        :param str heading:             Dialog heading.
        :param str message:             Message to display.
        :param str nolabel:             Label to put on the no button.
        :param str yeslabel:            Label to put on the yes button.
        :param str|None customlabel:    Label to put on the custom button.
        :param autoclose:               Milliseconds to autoclose dialog. (default=do not autoclose)

        :return: Returns True if 'Yes' was pressed, else False.
        :rtype: bool

        """

        if not nolabel:
            nolabel = "No"

        if not yeslabel:
            yeslabel = "Yes"

        self.print_heading(heading)
        question = "{} [{}]{} or [{}]{}:".format(message, yeslabel[0], yeslabel[1:], nolabel[0], nolabel[1:])
        if KodiStub.is_interactive:
            answer = self.read_input(question, Colors.Yellow)
            return yeslabel.lower().startswith(answer.lower())
        else:
            KodiStub.print_line(question, Colors.Yellow)
            return True

    # noinspection PyUnusedLocal
    def notification(self, heading, message, icon=NOTIFICATION_INFO, time=5000, sound=True):
        if icon == NOTIFICATION_WARNING:
            color = Colors.Yellow
        elif icon == NOTIFICATION_ERROR:
            color = Colors.Red
        else:
            color = Colors.White

        self.print_heading(heading, align_right=True, color=color)
        self.print_line(message, color=color)
        self.print_line("=" * 120, color)


# noinspection PyPep8Naming
class ListItem(KodiStub):

    # noinspection PyUnusedLocal
    def __init__(self, label="", label2="", path="", offscreen=False):
        """ ListItem class. Creates a new ListItem.

        label          : [opt] string or unicode -
        label2         : [opt] string or unicode - label2 text.
        iconImage      : [opt] string - icon filename.
        thumbnailImage : [opt] string - thumbnail filename.
        path           : [opt] string or unicode -

        :param str label:           Label1 text.
        :param str label2:          Label2 text.
        :param str path:            Listitem's path.
        :param bool offscreen:      Is an offscreen item?

        *Note, You can use the above as keywords for arguments and skip certain optional arguments.
           Once you use a keyword, all following arguments require the keyword.

        """

        super(ListItem, self).__init__()

        self.__info = dict()
        self.__art = dict()
        self.__type = None

        self.__label = label
        self.__info["*label1"] = label
        self.__label2 = label2
        self.__info["*label2"] = label2

        self.__path = path
        self.__subtitles = []

        # store properties
        self.__properties = {"path": path}

    def setIconImage(self, icon):  # NOSONAR
        raise DeprecationWarning("No more setIconImage: http://kodi.wiki/view/Jarvis_API_changes")

    def setThumbnailImage(self, thumbnail):  # NOSONAR
        raise DeprecationWarning("No more setThumbnailImage: http://kodi.wiki/view/Jarvis_API_changes")

    # noinspection PyShadowingBuiltins
    def setInfo(self, type, infoLabels):  # NOSONAR
        """ Sets the listitem's infoLabels.

        :param str type:                    Type of
        :param dict[str,str] infoLabels:    Pairs of { label: value }

        ============  ======================================
        Command name  Description
        ============  ======================================
        video         Video information
        music         Music information
        pictures      Pictures informanion
        game          Game information
        ============  ======================================

        See http://kodi.wiki/view/InfoLabels

        """

        self.__type = type
        self.__info.update(infoLabels)
        self.print_line("Updating infolabels with {}".format(infoLabels), verbose=True)

    def setArt(self, values):  # NOSONAR
        """ Sets the listitem's art

        :param dict[str,str] values:    Pairs of { label: value }.

        ==========  ======================================
        Label       Type
        ==========  ======================================
        thumb       image filename
        poster      image filename
        banner      image filename
        fanart      image filename
        clearart    image filename
        clearlogo   image filename
        landscape   image filename
        icon        image filename
        ==========  ======================================

        """

        self.__art.update(values)
        self.print_line("Updating artwork with {}".format(values), verbose=True)

    def setLabel(self, label):  # NOSONAR
        """ Sets the listitem's label.

        :param str label:   text string.

        """

        self.__label = label
        self.__art["label1"] = label
        self.print_line("Setting label1='{}'".format(label), verbose=True)

    def getLabel(self):
        """ Returns the listitem label.

        :return: label2
        :rtype: str

        """

        return self.__label

    def setLabel2(self, label):  # NOSONAR
        """ Sets the listitem's label.

        :param str label:   text string.

        """

        self.__label2 = label
        self.__art["label2"] = label
        self.print_line("Setting label2='{}'".format(label), verbose=True)

    def getLabel2(self):
        """ Returns the listitem label.

        :return: label2
        :rtype: str

        """

        return self.__label2

    def setSubtitles(self, subtitleFiles):  # NOSONAR
        """ Sets subtitles for this listitem.

        :param list[str] subtitleFiles: list with path to subtitle files

        """

        self.__subtitles = subtitleFiles
        for sub in subtitleFiles:
            self.print_line("Adding subtitles: {}".format(sub), verbose=True)

    def setProperty(self, key, value):  # NOSONAR
        """ Sets a listitem property, similar to an infolabel.

        :param str key:         Property name.
        :param str value:       Value of property.

        NOTE: Key is NOT case sensitive.

        """

        self.__properties[key.lower()] = value
        self.print_line("Adding property: {}: {}".format(key, value), verbose=True)

    def getProperty(self, key):  # NOSONAR
        """ Returns a listitem property as a string, similar to an infolabel.

        :param str key: property name.

        :return:

        NOTE: Key is NOT case sensitive.

        """

        return self.__properties.get(key.lower(), None)

    def getPath(self):  # NOSONAR
        """ Returns the path of this listitem.

        :return: The path of this listitem.
        :rtype: str
        """

        return self.__path or self.getProperty("path") or ""

    def __str__(self):
        if KodiStub.is_verbose:
            value = "%s [%s]\n" % (self.__label, self.__type or "")
            value = "%sInfoLabels\n" % (value,)
            keys = sorted(self.__info.keys())
            for key in keys:
                value = "%s    - %s: %s\n" % (value, key, self.__info[key])
            value = "%sProperties\n" % (value,)
            keys = sorted(self.__properties.keys())
            for key in keys:
                value = "%s    - %s: %s\n" % (value, key, self.__properties[key])

            return value
        else:
            return self.__label

    def __unicode__(self):
        return self.__str__()

    def __repr__(self):
        return self.__str__()


class DialogProgress(KodiStub):
    def __init__(self):
        """ Kodi's progress dialog class (Duh!) """

        super(DialogProgress, self).__init__()

    def create(self, heading, message=""):
        """
        Create and show a progress dialog.

        :param str heading:       dialog heading.
        :param str|None message:  line #1 multi-line text.

        It is preferred to only use line1 as it is actually a multi-line text.
        In this case line2 and line3 must be omitted.

        Use update() to update lines and progressbar.

        """

        self.print_heading(heading)
        self.print_line(message)

    def update(self, percent, message=""):
        """ Updates the progress dialog.

        :param int percent:         Percent complete. (0:100)
        :param str|None message:    Message to show.

        """

        self.print_line("{}{}%{}: {}".format(Colors.Yellow, percent, Colors.EndColor, message))

    def close(self):
        """ Close the progress dialog. """
        self.print_line("=" * 120, color=Colors.Yellow)

    def iscanceled(self):
        """ Checks progress is canceled. """
        return False


# noinspection PyArgumentList
class DialogProgressBG(KodiStub):
    def __init__(self):
        super(DialogProgressBG, self).__init__()

    def create(self, heading, message=""):
        """
        Create and show a progress background dialog.

        :param str heading:       dialog heading.
        :param str|None message:  line #1 multi-line text.

        It is preferred to only use line1 as it is actually a multi-line text.
        In this case line2 and line3 must be omitted.

        Use update() to update lines and progressbar.

        """

        self.print_heading(heading, align_right=True)
        self.print_line(message)

    def update(self, percent, heading="", message=""):
        """ Updates the progress dialog.

        :param int percent:         Percent complete. (0:100)
        :param str|None heading:    Dialog heading.
        :param str|None message:    Message to show.

        """

        if heading:
            self.print_line("{}{}%: {}{} - {}".format(Colors.Yellow, percent, heading, Colors.EndColor, message))
        else:
            self.print_line("{}{}%{}: {}".format(Colors.Yellow, percent, Colors.EndColor, message))

    def close(self):
        """ Close the progress dialog. """
        self.print_line("=" * 120, color=Colors.Yellow)

    # noinspection PyPep8Naming
    def isFinished(self):
        """ Checks progress is canceled. """
        return False



# SPDX-License-Identifier: GPL-3.0


class Helper(object):
    # noinspection PyUnusedLocal
    def __init__(self, protocol, drm=None):
        """Initialize InputStream Helper class

        :param str protocol:    The protocol to use.
        :param str drm:         The DRM that is required.

        """
        pass

    def check_inputstream(self):
        """ Check if Widevine is installed.

        :return: True if Widevine is installed
        :rtype: bool

        """

        return True



# SPDX-License-Identifier: GPL-3.0

import os
import re
import sys
import random

from sakee.colors import Colors


class KeyboardStub(object):
    def __init__(self):
        self.__id = random.random()
        self.__queue = []
        self.reset()

    def add_input(self, line):
        self.__queue.append(line)

    def clear_input(self):
        self.__queue = []

    def get_next_input(self):
        if len(self.__queue) == 0:
            return None

        return self.__queue.pop(0)

    def reset(self):
        self.__queue = []
        line = os.environ.get("KODI_STUB_INPUT", None)
        if line:
            self.__queue.append(line)


class KodiStub(object):
    __keyboard_stub = None

    is_interactive = os.environ.get("KODI_INTERACTIVE", "1") == "1"
    is_verbose = os.environ.get("KODI_STUB_VERBOSE", "0") == "1"

    def __init__(self):
        self.PY2 = sys.version_info[0] == 2
        self.PY3 = sys.version_info[0] == 3

    def get_keyboard_stub(self):
        if KodiStub.__keyboard_stub is None:
            KodiStub.__keyboard_stub = KeyboardStub()

        return KodiStub.__keyboard_stub

    def read_input(self, text, color=None):
        """ Reads user's input from the console

        :param str text:    Question for the input

        :return: the read input
        :rtype: str

        """

        text = "{} ".format(text)

        if color is not None:
            text = "{}{}{}".format(color, text, Colors.EndColor)

        # noinspection PyUnresolvedReferences
        return input(text) if self.PY3 else raw_input(text)

    @staticmethod
    def print_heading(text, align_right=False, color=Colors.Yellow):
        """ Aligns text over 120 chars

        :param str text:            The Text to align
        :param bool align_right:    Align right instead of left?

        :return: the aligned text
        :rrtype: str

        """

        if text == "":
            heading = "=" * 120
        elif not align_right:
            heading = "= {} {}".format(text, "=" * (120 - 3 - len(text)))
        else:
            heading = "{} {} =".format("=" * (120 - 3 - len(text)), text)

        KodiStub.print_line(heading, color=color)

    @staticmethod
    def print_line(line, color=None, verbose=False):
        """ Prints a full line including colors

        :param str line:        The line to print
        :param str color:       The color to print (use Color)
        :param bool verbose:    Is the line verbose data?

        """

        if verbose and not KodiStub.is_verbose:
            return

        if color:
            print(color + line + Colors.EndColor)
        else:
            print(line)

    def log_method(self, code_module, name, *args, **kwargs):
        """

        :param str code_module: The module that called this method
        :param str name:        The method that was callled
        :param *args:           The arguments that were passed
        :param **kwargs:        The keyword arguments that where passed

        :return: None

        """

        KodiStub.print_line("Call to missing: {0}.{1}".format(code_module, name), color=Colors.Blue, verbose=True)
        if not self.is_verbose:
            return

        if not args and not kwargs:
            print("-> %s.%s()" % (code_module, name,))
        if args and not kwargs:
            print("-> %s.%s(args=%s)" % (code_module, name, args))
        if args and kwargs:
            print("-> %s.%s(args=%s, kwargs=%s)" % (code_module, name, args, kwargs))
        if not args and kwargs:
            print("-> %s.%s(kwargs=%s)" % (code_module, name, kwargs))

    @staticmethod
    def replace_colors(color_tag):
        """ Replace the Kodi color tags with actual tags.

        :param str color_tag: The text that contains color tags

        :rtype: str
        :return: The text with actual ASCII color codes

        """

        def __color_replacer(color_input):
            color_name = color_input.group(1)
            if color_name == "gold":
                return Colors.Yellow
            elif color_name == "dimgray":
                return Colors.DarkGrey
            elif color_name == "aqua":
                return Colors.LightBlue
            elif color_name == "red":
                return Colors.Red
            else:
                return Colors.EndColor

        color_tag = re.sub(r'\[COLOR (\w+)]', __color_replacer, color_tag)
        color_tag = color_tag.replace("[/COLOR]", Colors.EndColor)
        return color_tag

    def __str__(self):
        return self.__class__.__name__

    def __getattr__(self, name):
        """ Logs any missing methods calls

        :param str name:    The name of the attribute
        :return: object

        """

        def method(*args, **kwargs):
            return self.log_method(self.__class__.__name__, name, *args, **kwargs)
        return method



# SPDX-License-Identifier: GPL-3.0

import os
import io
import re

from sakee import addoninfo
from sakee.stub import KodiStub


# noinspection PyPep8Naming,PyShadowingBuiltins
class Addon(KodiStub):
    __settings = {}

    def __init__(self, id=None):
        super(Addon, self).__init__()

        self.__add_on_id = id
        # Get some data from the path of the calling file
        paths = addoninfo.get_add_on_info_from_calling_script(id)
        self.__kodi_home_path = paths.kodi_home_path
        self.__add_on_id = paths.add_on_id
        self.__add_on_path = paths.add_on_path
        self.__add_on_profile_path = os.path.join(paths.kodi_profile_path, "addon_data", self.__add_on_id)

        # actual live data
        self.__version = None
        self.__name = None
        self.__author = None
        self.__icon = None
        self.__summary = None
        self.__news = None
        self.__disclaimer = None
        self.__description = None
        self.__load_add_on_xml()
        self.__localization = self.__get_strings()
        self.__settings = self.__get_settings()

    def getSetting(self, id):  # NOSONAR
        """ Returns the value of a setting as a unicode string.

        :param str id:  Id of the setting that the module needs to access.

        :return: The value of a setting as a unicode string.
        :rtype: str

        """
        if id in self.__settings:
            return self.__settings[id]
        else:
            return ''

    def getSettingBool(self, id):
        """ Returns the value of a setting as a boolean.

        :param str id:  Id of the setting that the module needs to access.

        :return: The value of a setting as a boolean.
        :rtype: bool

        """
        return self.getSetting(id).lower() == "true"

    def getSettingInt(self, id):
        """ Returns the value of a setting as an integer.

        :param str id:  Id of the setting that the module needs to access.

        :return: The value of a setting as an integer.
        :rtype: int

        """

        return int(self.getSetting(id) or 0)

    def getSettingNumber(self, id):
        """ Returns the value of a setting as a floating point number.

        :param str id:  Id of the setting that the module needs to access.

        :return: The value of a setting as a floating point number.
        :rtype: float

        """

        return float(self.getSetting(id) or 0.0)

    def getSettingString(self, id):
        """ Returns the value of a setting as a string.

        :param str id:  Id of the setting that the module needs to access.

        :return: The value of a setting as a string.
        :rtype: float

        """

        return str(self.getSetting(id) or "")

    def setSetting(self, id, value):  # NOSONAR
        """ Sets a script setting.

        :param str id:       Id of the setting that the module needs to access.
        :param str value:    Value of the setting.

        """

        self.__settings[id] = value

    def setSettingBool(self, id, value):  # NOSONAR
        """ Sets a script setting.

        :param str id:       Id of the setting that the module needs to access.
        :param bool value:    Value of the setting.

        """

        self.__settings[id] = "true" if value else "false"
        return True

    def setSettingInt(self, id, value):  # NOSONAR
        """ Sets a script setting.

        :param str id:       Id of the setting that the module needs to access.
        :param int value:    Value of the setting.

        """

        self.__settings[id] = str(value)
        return True

    def setSettingNumber(self, id, value):  # NOSONAR
        """ Sets a script setting.

        :param str id:       Id of the setting that the module needs to access.
        :param float value:  Value of the setting.

        """

        self.__settings[id] = str(value)
        return True

    def setSettingString(self, id, value):  # NOSONAR
        """ Sets a script setting.

        :param str id:       Id of the setting that the module needs to access.
        :param str value:  Value of the setting.

        """

        self.__settings[id] = value
        return True

    def getAddonInfo(self, id):  # NOSONAR
        """ Returns the value of an addon property as a string.

        :param str id:  Id of the property that the module needs to access.

        :return: Returns the value of an addon property as a string.
        :rtype: str

        Possible options are: author, changelog, description, disclaimer, fanart, icon, id,
                              name, path, profile, stars, summary, type, version
        """
        id = id.lower()
        # missing: starts - type

        if id == "author":
            return self.__author
        elif id == "changelog":
            return self.__news
        elif id == "description":
            return self.__description
        elif id == "disclaimer":
            return self.__disclaimer
        elif id == "fanart":
            return self.__fanart
        elif id == "icon":
            return self.__icon
        elif id == "id":
            return self.__add_on_id
        elif id == "name":
            return self.__name
        elif id == "path":
            return self.__add_on_path
        elif id == "profile":
            return self.__add_on_profile_path
        elif id == "summary":
            return self.__summary
        elif id == "version":
            return self.__version

        raise ValueError("Cannot find info '%s'" % (id,))

    def getLocalizedString(self, id):  # NOSONAR
        """ Returns an addon's localized 'unicode string'.

        :param int id:      Id# for string you want to localize.

        :return: Localized 'unicode string'
        :rtype: str

        """

        return self.__localization.get(id, "Translated {}".format(id))

    def openSettings(self):  # NOSONAR
        self.print_heading("Add-on settings")
        for setting, value in self.__settings.items():
            self.print_line("{}:{}".format(setting, value), verbose=True)

    def __repr__(self):
        return repr(self.__settings)

    def __get_strings(self):
        english = os.path.join(self.__add_on_path, "resources", "language", "resource.language.en_gb", "strings.po")
        if not os.path.isfile(english):
            return {}

        with io.open(english, "r", encoding='utf-8') as fp:
            english_data = fp.read()

        translations = {}
        if not english_data:
            return translations

        # Take into account that there could be multi-line translations.
        strings = re.findall(
            r'msgctxt "#(\d+)"\W+msgid ((?:"[^\n\r]*"\W{1,2})+)msgstr ((?:"[^\n\r]*"\W{1,2})+)',
            english_data, flags=re.IGNORECASE)

        for string in strings:
            translation = string[1]
            # See if there was a translation present, if so, use it.
            if string[2].strip('\n\r"'):
                translation = string[2]
            parts = translation.split('"')
            msgs = [t for t in parts if t.strip()]
            translation = "".join(msgs)
            translations[int(string[0])] = translation

        return translations

    def __filter_lang_xmltag(self, tag, xml_content):
        tag_matches = re.findall(r'<' + tag + r'(?:\s*lang=\"([a-zA-Z-_]+)\")?\s*>(.*?)</' + tag + '>', xml_content, flags=re.DOTALL)
        if tag_matches:
            for match in tag_matches:
                if match[0].startswith('en'):
                    return match[1]
            else:
                return tag_matches[0][1]
        return None

    def __load_add_on_xml(self):
        add_on_xml = os.path.join(self.__add_on_path, "addon.xml")

        with io.open(add_on_xml, encoding='utf-8') as fp:
            xml_content = fp.read()
            self.__version = re.findall(r'<addon.*?version="([^"]*)', xml_content, flags=re.DOTALL)[0]
            self.__add_on_id = re.findall(r'addon.*?id="([^"]+)"', xml_content, flags=re.DOTALL)[0]
            self.__name = re.findall(r'name="([^"]+)"', xml_content)[0]

            self.__description = self.__filter_lang_xmltag('description', xml_content)
            self.__disclaimer = self.__filter_lang_xmltag('disclaimer', xml_content)
            self.__summary = self.__filter_lang_xmltag('summary', xml_content)

            author_matches = re.findall(r'<addon.*?provider-name="([^"]+)', xml_content, flags=re.DOTALL)
            if author_matches:
                self.__author = author_matches[0]

            news_matches = re.findall(r'<news>(.*?)</news>', xml_content, flags=re.DOTALL)
            if news_matches:
                self.__news = news_matches[0]

            fanart_matches = re.findall(r'<fanart>(.*?)</fanart>', xml_content, flags=re.DOTALL)
            if fanart_matches:
                self.__fanart = os.path.join(self.__add_on_path, fanart_matches[0])

            icon_matches = re.findall(r'<icon>(.*?)</icon>', xml_content, flags=re.DOTALL)
            if icon_matches:
                self.__icon = os.path.join(self.__add_on_path, icon_matches[0])

    def __get_settings(self):
        if self.__add_on_id in Addon.__settings:
            settings = Addon.__settings[self.__add_on_id]
            return settings

        # load defaults
        settings = {}
        settings_xml = os.path.join(self.__add_on_path, "resources", "settings.xml")
        if not os.path.isfile(settings_xml):
            Addon.__settings[self.__add_on_id] = settings
            return settings

        with io.open(settings_xml, encoding='utf-8') as fp:
            default_xml = fp.read()

        setting_regex = r'id="([^"]+)"[^>]*default="([^"]*)"'
        results = re.findall(setting_regex, default_xml)
        if not results:
            results = re.findall(r'setting id="(.*?)".*?<default>(.*?)<', default_xml, re.DOTALL)
        settings = {}
        for result in results:
            settings[result[0]] = result[1]

        user_xml = os.path.join(self.__add_on_profile_path, "settings.xml")
        if os.path.isfile(user_xml):
            with io.open(user_xml, encoding="utf-8") as fp:
                user_xml = fp.read()

            setting_regex = r'id="([^"]+)"[^>]*value="([^"]*)"'
            results = re.findall(setting_regex, user_xml)
            if not results:
                results = re.findall(r'id="([^"]+)"[^>]*>([^<]+)<', user_xml)

            for result in results:
                # if result[0] in settings:  -> We need to keep all settings in the settings.xml
                settings[result[0]] = result[1]

        Addon.__settings[self.__add_on_id] = settings
        return settings



# SPDX-License-Identifier: GPL-3.0

import io
import json
import os
import signal
import time

from sakee import addoninfo
from sakee.colors import Colors
from sakee.internalplayer import KodiInteralPlayer
from sakee.stub import KodiStub
from xbmcgui import ListItem

DRIVE_NOT_READY = 1
ENGLISH_NAME = 2
ISO_639_1 = 0
ISO_639_2 = 1
LOGDEBUG = 0
LOGERROR = 4
LOGFATAL = 6
LOGINFO = 1
LOGNONE = 7
LOGNOTICE = 2
LOGSEVERE = 5
LOGWARNING = 3
PLAYLIST_MUSIC = 0
PLAYLIST_VIDEO = 1
SERVER_AIRPLAYSERVER = 2
SERVER_EVENTSERVER = 6
SERVER_JSONRPCSERVER = 3
SERVER_UPNPRENDERER = 4
SERVER_UPNPSERVER = 5
SERVER_WEBSERVER = 1
SERVER_ZEROCONF = 7
TRAY_CLOSED_MEDIA_PRESENT = 96
TRAY_CLOSED_NO_MEDIA = 64
TRAY_OPEN = 16


# noinspection PyPep8Naming
class Monitor(KodiStub):
    def __init__(self):
        """ Creates a Dummy Kodi Monitor class """

        super(Monitor, self).__init__()
        self.__abort = False

        # noinspection PyUnusedLocal
        def stop_requested(signum, frame):
            self.__abort = True

        # Requires PyCharm to set the option of "Emulate terminal in output window" to work
        signal.signal(signal.SIGINT, stop_requested)

    def abortRequested(self):  # NOSONAR
        """ Returns True if abort has been requested.

        :return: True if abort has been requested.
        :rtype: bool

        """

        return self.__abort

    def waitForAbort(self, seconds=0.0):  # NOSONAR
        """ Block until abort is requested, or until timeout occurs. If an abort requested
        have already been made, return immediately.

        :param float seconds:  timeout in seconds. Default: no timeout.

        :return: True when abort have been requested, False if a timeout is given and
        the operation times out.
        :rtype: bool

        """

        deadline = time.time() + seconds
        while time.time() < deadline:
            if self.__abort:
                return True
            time.sleep(0.1)  # Sleep 100ms

        return False


# noinspection PyPep8Naming
class Keyboard(KodiStub):
    def __init__(self, line="", heading="", hidden=False):
        """

        :param str line:        The default line to show?
        :param str heading:     The heading of the dialog
        :param bool hidden:     Hidden input?

        """

        super(Keyboard, self).__init__()

        self.log_method("xbmc.Keyboard", "__init__", line, heading, hidden)
        self.__confirmed = False
        self.__line = line
        self.__heading = heading
        self.__hidden = hidden
        self.__input = ""

    # noinspection PyUnusedLocal
    def doModal(self, autoclose=0):  # NOSONAR
        """ Show keyboard and wait for user action.

        :param int autoclose: milliseconds to autoclose dialog. (default=do not autoclose)

        """

        if not self.is_interactive:
            self.__confirmed = True
            keyboard = self.get_keyboard_stub()
            self.__input = keyboard.get_next_input()
            return

        KodiStub.print_heading(self.__heading)
        try:
            answer = self.read_input(
                "Please provide keyboard input [{}]?".format(self.__line), color=Colors.Yellow)
            if not answer:
                answer = self.__line
            self.__confirmed = True
        except EOFError:
            answer = ""
            self.__confirmed = False

        self.__input = answer

    def getText(self):  # NOSONAR
        """ Returns the user input as a string.

        :return: Returns the user input as a string.
        :rtype: str

        This will always return the text entry even if you cancel the keyboard. Use
        the isConfirmed() method to check if user cancelled the keyboard.

        """

        return self.__input

    def isConfirmed(self):  # NOSONAR
        """ Returns False if the user cancelled the input.

        :return: Returns False if the user cancelled the input.
        :rtype: bool

        """

        return self.__confirmed

    def setDefault(self, line):  # NOSONAR
        """ Set the default text entry.

        :param str line:   The default line to show?

        """

        self.__line = line

    def setHeading(self, heading):  # NOSONAR
        """ Set the keyboard heading.

        :param str heading: Keyboard heading

        """

        self.__heading = heading

    def setHiddenInput(self, hidden):  # NOSONAR
        """ Allows hidden text entry.

        :param bool hidden:     True for hidden text entry.

        """

        self.__hidden = hidden


# noinspection PyArgumentList,PyPep8Naming
class PlayList(KodiStub):
    def __init__(self, playList):  # NOSONAR
        """ Playlist object

        :param int playList:    The type of playlist

        PLAYLIST_MUSIC = 13
        PLAYLIST_VIDEO = 14

        """

        self.__play_list_type = playList
        self.__items = []

        super(PlayList, self).__init__()

    def getPlayListId(self):
        """ Returns the type of playlist

        :return: The type of playlist
        :rtype: int

        """
        return self.__play_list_type

    def clear(self):
        """ Clear all items in the playlist. """
        self.__items = []

    def getposition(self):
        """ Returns the position of the current song in this playlist.

        :return: Position of the current song
        :rtype: int
        """
        self.log_method("xbmc.PlayList", "getposition")
        return 0

    def add(self, url, listitem=None, index=None):
        """`Adds a new file to the playlist.

        :param str url:                 Filename or url to add.
        :param ListItem|None listitem:  Used with setInfo() to set different infolabels.
        :param int|None index:          Position to add playlist item. (default=end)
        """

        if index is not None:
            self.__items.insert(index, (url, listitem))
        else:
            self.__items.append((url, listitem))

    def __len__(self):
        return len(self.__items)

    def __getitem__(self, i):
        return self.__items[i]


# noinspection PyPep8Naming,PyArgumentList
class Player(KodiStub):
    def __init__(self):
        super(Player, self).__init__()

        # register this player with the main Kodi player
        KodiInteralPlayer.instance().register_player(self)

    def __del__(self):
        # unregister
        KodiInteralPlayer.instance().unregister_player(self)

    def getAvailableAudioStreams(self):  # NOSONAR
        """ Returns the available audio stream names.

        :return: The available audio streams.
        :rtype: List[str]

        """

        return []  # Not implemented

    def getAvailableSubtitleStreams(self):  # NOSONAR
        """ Returns the available subtitle stream names.

        :return: The available subtitle streams.
        :rtype: List[str]

        """

        return []  # Not implemented

    def getAvailableVideoStreams(self):  # NOSONAR
        """ Returns the available video stream names.

        :return: The available video streams.
        :rtype: List[str]

        """

        return []  # Not implemented

    def getMusicInfoTag(self):
        """ Return the music info tag.

        :return: Returns the MusicInfoTag of the current playing 'Song'.
        :rtype: MusicInfoTag|None

        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        return None  # Not implemented

    def getPlayingFile(self):  # NOSONAR
        """ Returns the current playing file as a string.

        :return: The filename of the playing item.
        :rtype: str

        """

        return KodiInteralPlayer.instance().file

    def getRadioRDSInfoTag(self):
        """ Return the Radio RDS info tag.

        :return: Returns the RadioRDSInfoTag of the current playing 'Radio Song if. present'.
        :rtype: RadioRDSInfoTag|None
        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        return None  # Not implemented

    def getSubtitles(self):
        """ Return the subtitle stream name.

        :return: Stream name
        :rtype: str

        """

        return ''  # Not implemented

    def getTime(self):
        """ Return the current playing time.

        :return: Returns the current time of the current playing media as fractional seconds.
        :rtype: int
        """

        return KodiInteralPlayer.instance().current_time

    def getTotalTime(self):
        """ Return the total playing time.

        :return: Returns the total time of the current playing media in seconds. This is only accurate to the full second.
        :rtype: int

        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        return KodiInteralPlayer.instance().total_time

    def getVideoInfoTag(self):
        """ Return the video info tag.

        :return: The VideoInfoTag of the current playing Movie.
        :rtype: VideoInfoTag|None

        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        return None  # Not implemented

    def isExternalPlayer(self):
        """ Check for external player.

        :return: True if kodi is playing using an external player.
        :rtype: bool

        """

        return False

    def isPlaying(self):  # NOSONAR
        """ Check Kodi is playing something.

        :return: True if Kodi is playing a file.
        :rtype: bool

        """

        return KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PLAYING

    def isPlayingAudio(self):  # NOSONAR
        """ Check for playing audio.

        :return: True if Kodi is playing an audio file.
        :rtype: bool

        """

        return KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PLAYING

    def isPlayingRDS(self):  # NOSONAR
        """ Check for playing radio data system (RDS).

        :return: True if kodi is playing a radio data system (RDS).
        :rtype: bool

        """

        return KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PLAYING

    def isPlayingVideo(self):  # NOSONAR
        """ Check for playing video.

        :return: True if Kodi is playing a video.
        :rtype: bool

        """

        return KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PLAYING

    def pause(self):  # NOSONAR
        """ Pause or resume playing if already paused. """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            return

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PLAYING:
            KodiInteralPlayer.instance().status = KodiInteralPlayer.STATUS_PAUSED
            self.onPlayBackPaused()
            return

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_PAUSED:
            KodiInteralPlayer.instance().status = KodiInteralPlayer.STATUS_PLAYING
            self.onPlayBackResumed()
            return

    # noinspection PyUnusedLocal
    def play(self, item=None, listitem=None, windowed=False, startpos=-1):
        """ Play an item.

        :param str|None item:            Filename, url or playlist
        :param ListItem|None listitem:   Used with setInfo() to set different infolabels.
        :param bool windowed:            True=play video windowed, False=play users preference.(default)
        :param int startpos:             Starting position when playing a playlist. Default = -1

        If item is not given then the Player will try to play the current item in the current playlist.

        You can use the above as keywords for arguments and skip certain optional arguments.
        Once you use a keyword, all following arguments require the keyword.

        """
        # Stop playing the current file (if any)
        self.stop()
        KodiInteralPlayer.instance().play(item)

    def playnext(self):  # NOSONAR
        """ Play next item in playlist."""

        pass  # Not implemented

    def playprevious(self):  # NOSONAR
        """ Play previous item in playlist."""

        pass  # Not implemented

    def playselected(self, selected):  # NOSONAR
        """ Set Audio Stream.

        :param int selected:            Item to select

        """

        pass  # Not implemented

    def seekTime(self, seekTime):  # NOSONAR
        """ Seek time.

        Seeks the specified amount of time as fractional seconds.
        The time specified is relative to the beginning of the currently. playing media file.

        :param int seekTime:            Time to seek as fractional seconds

        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        KodiInteralPlayer.instance().current_time = seekTime
        self.onPlayBackSeek(seekTime, 0)

    def setAudioStream(self, stream):  # NOSONAR
        """ Set Audio Stream.

        :param int stream:              Audio stream to select for play

        """

        pass  # Not implemented

    def setSubtitles(self, subtitleFile):  # NOSONAR
        """ Set subtitle file and enable subtitles.

        :param str subtitleFile:        File to use as source of subtitles

        """

        pass  # Not implemented

    def setSubtitleStream(self, stream):  # NOSONAR
        """ Set Subtitle Stream.

        :param int stream:              Subtitle stream to select for play

        """

        pass  # Not implemented

    def setVideoStream(self, stream):  # NOSONAR
        """ Set Video Stream.

        :param int stream:              Video stream to select for play

        """

        pass  # Not implemented

    def showSubtitles(self, visible):  # NOSONAR
        """ Enable / disable subtitles.

        :param bool visible:            True for visible subtitles.

        """

        pass  # Not implemented

    def stop(self):  # NOSONAR
        """ Stop playing."""

        KodiInteralPlayer.instance().status = KodiInteralPlayer.STATUS_STOPPED
        KodiInteralPlayer.instance().current_time = 0
        KodiInteralPlayer.instance().total_time = 0
        KodiInteralPlayer.instance().file = None
        KodiInteralPlayer.instance().stop_playback()

    # noinspection PyUnusedLocal
    def updateInfoTag(self, item):  # NOSONAR
        """ Update info labels for currently playing item.

        :param ListItem item:           ListItem with new info

        """

        if KodiInteralPlayer.instance().status == KodiInteralPlayer.STATUS_STOPPED:
            raise Exception('Player is not playing a file.')

        # Not implemented

    def onAVChange(self):  # NOSONAR
        """ onAVChange method.
        Will be called when Kodi has a video, audio or subtitle stream. Also happens when the stream changes.

        """

        self.print_line('Invoked onAVChange()', verbose=True)

    def onAVStarted(self):  # NOSONAR
        """ onAVStarted method.

        Will be called when Kodi has a video or audiostream.

        """

        self.print_line('Invoked onAVStarted()', verbose=True)

    def onPlaybackEnded(self):  # NOSONAR
        """ onPlaybackEnded method.

        Will be called when Kodi stops playing a file.

        """
        self.print_line('Invoked onPlaybackEnded()', verbose=True)

    def onPlayBackError(self):  # NOSONAR
        """ onPlayBackError method.

        Will be called when playback stops due to an error.

        """

        self.print_line('Invoked onPlayBackError()', verbose=True)

    def onPlayBackPaused(self):  # NOSONAR
        """ onPlayBackPaused method.

        Will be called when user pauses a playing file.

        """

        self.print_line('Invoked onPlayBackPaused()', verbose=True)

    def onPlayBackResumed(self):  # NOSONAR
        """ onPlayBackResumed method.

        Will be called when user resumes a paused file.

        """

        self.print_line('Invoked onPlayBackResumed()', verbose=True)

    def onPlayBackSeek(self, time, seekOffset):  # NOSONAR
        """ onPlayBackSeek method.
        Will be called when user seeks to a time.

        :param int time:            Time to seek to
        :param int seekOffset:      ?

        """
        self.print_line('Invoked onPlayBackSeek(%d, %d)' % (time, seekOffset), verbose=True)

    def onPlayBackSeekChapter(self, chapter):  # NOSONAR
        """ onPlayBackSeekChapter method.

        Will be called when user performs a chapter seek.

        :param int chapter:         Chapter to seek to

        """

        self.print_line('Invoked onPlayBackSeekChapter(%d)' % chapter, verbose=True)

    def onPlayBackSpeedChanged(self, speed):  # NOSONAR
        """ onPlayBackSpeedChanged method.

        Will be called when players speed changes (eg. user FF/RW).

        Negative speed means player is rewinding, 1 is normal playback speed.

        :param int speed:           Current speed of player

        """

        self.print_line('Invoked onPlayBackSpeedChanged(%d)' % speed, verbose=True)

    def onPlayBackStarted(self):  # NOSONAR
        """ onPlayBackStarted method.

        Will be called when Kodi player starts. Video or audio might not be available at this point.

        Use onAVStarted() instead if you need to detect if Kodi is actually playing a media file
        (i.e, if a stream is available).

        """

        self.print_line('Invoked onPlayBackStarted()', verbose=True)

    def onPlayBackStopped(self):  # NOSONAR
        """ onPlayBackStopped method.

        Will be called when user stops Kodi playing a file.

        """

        self.print_line('Invoked onPlayBackStopped()', verbose=True)

    def onQueueNextItem(self):  # NOSONAR
        """ onQueueNextItem method.

        Will be called when user queues the next item.

        """

        self.print_line('Invoked onQueueNextItem()', verbose=True)


# noinspection PyPep8Naming
def executeJSONRPC(jsonrpccommand):  # NOSONAR
    """ Execute an JSONRPC command.

    :param str jsonrpccommand:   jsonrpc command to execute.

    :return: jsonrpc return string
    :rtype: str

    See https://codedocs.xyz/xbmc/xbmc/namespace_j_s_o_n_r_p_c.html
    """
    from sakee.sakejsonrpc import JsonRpcApi

    json_data = json.loads(jsonrpccommand)
    try:
        # Implement some methods for real
        return json.dumps(JsonRpcApi().handle(json_data))

    except NotImplementedError:
        # Fallback to stubs
        pass

    json_responses = os.environ.get("KODI_STUB_RPC_RESPONSES")
    if not json_responses:
        raise ValueError(
            "Could not find JSON Response folder. Use the environment variable KODI_STUB_RPC_RESPONSES to set one")

    path = "{}.json".format(os.path.join(json_responses, json_data["method"].lower()))
    if os.path.isfile(path):
        with io.open(path, mode='r', encoding='utf-8') as fd:
            stub_content = json.loads(fd.read())
            if isinstance(stub_content, dict):
                return json.dumps(stub_content)
            try:
                return json.dumps(next(stub.get('response')
                                       for stub in stub_content
                                       if json_data.get('params')
                                       and stub.get('request', {}).get('method') == json_data.get('method')
                                       and stub.get('request', {}).get('params') == json_data.get('params')))
            except StopIteration:
                pass

    return '{"id":1,"jsonrpc":"2.0","result":"OK"}'


# noinspection PyPep8Naming
def translatePath(path):  # NOSONAR
    """ Returns the translated path.

    :param str path:    Path to format

    :return: Translated path
    :rtype: str

    See http://kodi.wiki/view/Special_protocol

    E.g:
        special://home/ is mapped to: kodi/
        special://profile/ is mapped to: kodi/userdata

    Or in portable:
        special://home/ is mapped to: kodi/portable_data/
        special://profile/ is mapped to: kodi/portable_data/userdata

    """

    KodiStub.print_line("Warning: 'xbmc.translatePath' was moved to xbmcvfs in Kodi Matrix (API version 19.0)", color=Colors.Red)
    import xbmcvfs
    return xbmcvfs.translatePath(path)


def executebuiltin(function):
    """ Execute a built in Kodi function.

    :param str function:    builtin function to execute.

    See: http://kodi.wiki/view/List_of_Built_In_Functions

    """

    KodiStub.print_line("Executebuiltin: {0}".format(function), color=Colors.Blue)


# noinspection PyPep8Naming
def getCondVisibility(condition):  # NOSONAR
    """ Get visibility conditions

    :param str condition:   condition to check

    :return: True (1) or False (0) as a bool
    :rtype: int

    List of Conditions - http://kodi.wiki/view/List_of_Boolean_Conditions

    """
    result = False
    if condition.startswith('System.HasAddon('):
        add_on_id = condition.replace('System.HasAddon(', '').replace(')', '').strip('"')
        add_ons = os.listdir(os.path.join(__add_on_info.kodi_home_path, "addons"))

        parent_path = os.path.join(__add_on_info.kodi_home_path, "..", "addons")
        if os.path.isdir(parent_path):
            add_ons += os.listdir(parent_path)

        result = add_on_id in add_ons

    elif condition == 'system.platform.windows':
        result = os.name == "nt"

    elif condition == 'system.platform.linux':
        result = os.name == "posix"

    elif condition == 'system.platform.xbox' or condition == 'system.platform.uwp':
        result = False

    else:
        KodiStub.print_line("Missing condition: {}".format(condition), color=Colors.Yellow)

    KodiStub.print_line("Condition: {0}={1}".format(condition, result), color=Colors.Blue, verbose=True)
    return 1 if result else 0


# noinspection PyPep8Naming
def getInfoLabel(infoTag):  # NOSONAR
    """ Get a info label

    :param str infoTag:  infoTag for value you want returned.

    :return: InfoLabel as a string
    :rtype: str

    List of InfoTags - http://kodi.wiki/view/InfoLabels
    """

    if infoTag.lower() == "system.buildversion":
        return "19.0 Git:20200626-xxxxxxxxxx"

    return "InfoLabel:{}".format(infoTag)


# noinspection PyPep8Naming,PyShadowingBuiltins,PyUnusedLocal
def getLanguage(format=ENGLISH_NAME, region=False):  # NOSONAR
    """ Get the active language.

    :param int format:          Format of the returned language string (see table)
    :param bool region:         append the region delimited by "-" of the language
                                (setting) to the returned language string

    :return: The active language as a string
    :rtype: str|unicode

    ==================  ========================================================
    Value               Description
    ==================  ========================================================
    xbmc.ISO_639_1      Two letter code as defined in ISO 639-1
    xbmc.ISO_639_2      Three letter code as defined in ISO 639-2/T
                        or ISO 639-2/B
    xbmc.ENGLISH_NAME   Full language name in English (default)
    ==================  ========================================================

    """

    if format == ISO_639_1:
        return "en" if not region else "en-gb"
    if format == ISO_639_2:
        return "eng" if not region else "eng-gbr"

    return "English"


def log(msg, level=0):
    """ Write a string to Kodi's log file and the debug window.

    :param str msg:      Text to output.
    :param int level:    Log level to output at. (default=LOGDEBUG)

    ==================  ========================================================
    Value               Description
    ==================  ========================================================
    xbmc.LOGDEBUG       In depth information about the status of Kodi. This information can pretty much only be deciphered by a developer or long time Kodi power user.
    xbmc.LOGINFO        Something has happened. It's not a problem, we just thought you might want to know. Fairly excessive output that most people won't care about.
    xbmc.LOGNOTICE      Similar to INFO but the average Joe might want to know about these events. This level and above are logged by default.
    xbmc.LOGWARNING     Something potentially bad has happened. If Kodi did something you didn't expect, this is probably why. Watch for errors to follow.
    xbmc.LOGERROR       This event is bad. Something has failed. You likely noticed problems with the application be it skin artifacts, failure of playback a crash, etc.
    xbmc.LOGFATAL       We're screwed. Kodi is about to crash.
    ==================  ========================================================

    Note: You can use the above as keywords for arguments and skip certain optional arguments.
    Once you use a keyword, all following arguments require the keyword.

    """

    if level == LOGERROR:
        print(Colors.Red + msg + Colors.EndColor)
    elif level == LOGWARNING:
        print(Colors.Yellow + msg + Colors.EndColor)
    else:
        print(msg)


__add_on_info = addoninfo.get_add_on_info_from_calling_script(print_info=True)



# SPDX-License-Identifier: GPL-3.0

import threading

from sakee.stub import KodiStub


class KodiInteralPlayer(KodiStub):  # NOSONAR
    STATUS_INIT = 'init'
    STATUS_STOPPED = 'stopped'
    STATUS_PLAYING = 'playing'
    STATUS_PAUSED = 'paused'
    __kodi_player = None

    @staticmethod
    def instance():
        """ The Kodi player instance

        :return: The stub for the internal Kodi player
        :rtype: KodiInteralPlayer

        """

        if KodiInteralPlayer.__kodi_player is None:
            KodiInteralPlayer.__kodi_player = KodiInteralPlayer()

        return KodiInteralPlayer.__kodi_player

    def __init__(self):
        super(KodiInteralPlayer, self).__init__()

        self.status = KodiInteralPlayer.STATUS_STOPPED
        self.file = None
        self.current_time = 0
        self.total_time = 0

        self._stop_event = threading.Event()

        # Keep track of players
        self.__players = []

        # The play thread
        self.__play_thread = None
        self.__kill_play_thread = False

    # noinspection PyUnusedLocal
    def play_resolved_item(self, path, item):  # NOSONAR
        """ Sets the resolved item to play

        :param str path:
        :param ListItem item:

        """

        self.file = path
        self.play(path)

    def play(self, path):
        self.file = path
        self.status = KodiInteralPlayer.STATUS_INIT
        self.total_time = 5  # 5 seconds
        self._stop_event.clear()

        # Start the playback simulation thread
        self.__play_thread = threading.Thread(target=self.__loop)
        self.__play_thread.start()

    def stop_playback(self, force=False):
        """ Stops playback. If `force=True` no stop events are raised.

        :param bool force:      Force stops the player.

        """

        if self.__play_thread is None:
            return

        if force and self.__play_thread:
            self.__kill_play_thread = True
            return

        self._stop_event.set()
        self.__play_thread.join(2)
        self.__play_thread = None
        self.__kill_play_thread = False

    def register_player(self, player):
        """ Register a xbmc.Player instance

        :param Player player: The player to register

        """

        self.__players.append(player)

    def unregister_player(self, player):
        """ Unregister a xbmc.Player instance

        :param Player player: The player to unregister

        """

        self.__players.remove(player)

    def __loop(self):
        """ Background playback loop. """
        from datetime import timedelta

        for player in self.__players:
            player.onPlayBackStarted()

        while not self._stop_event.wait(1):
            if self.__kill_play_thread:
                return

            KodiStub.print_line(
                'Player: [{status}] {pos}/{total}'.format(
                    status=self.status,
                    pos=timedelta(seconds=self.current_time),
                    total=timedelta(seconds=self.total_time)
                ), verbose=True)

            if self.status == KodiInteralPlayer.STATUS_INIT:
                self.status = KodiInteralPlayer.STATUS_PLAYING
                self.current_time = 0
                for player in self.__players:
                    player.onAVStarted()
                    player.onAVChange()
                continue

            if self.status == KodiInteralPlayer.STATUS_PLAYING:
                self.current_time += 1
                if self.current_time > self.total_time:
                    for player in self.__players:
                        player.stop()

            if self.status == KodiInteralPlayer.STATUS_STOPPED:
                break

        for player in self.__players:
            player.onPlayBackStopped()



# SPDX-License-Identifier: GPL-3.0
from sakee.colors import Colors
from sakee.stub import KodiStub
from xbmc import ListItem


class HandleInfo(object):
    # Mapping from SORT_METHOD_ID to names
    __sort_method_names = {
        0: "SORT_METHOD_NONE",
        1: "SORT_METHOD_LABEL",
        2: "SORT_METHOD_LABEL_IGNORE_THE",
        3: "SORT_METHOD_DATE",
        4: "SORT_METHOD_SIZE",
        5: "SORT_METHOD_FILE",
        6: "SORT_METHOD_DRIVE_TYPE",
        7: "SORT_METHOD_TRACKNUM",
        8: "SORT_METHOD_DURATION",
        9: "SORT_METHOD_TITLE",
        10: "SORT_METHOD_TITLE_IGNORE_THE",
        11: "SORT_METHOD_ARTIST",
        13: "SORT_METHOD_ARTIST_IGNORE_THE",
        14: "SORT_METHOD_ALBUM",
        15: "SORT_METHOD_ALBUM_IGNORE_THE",
        16: "SORT_METHOD_GENRE",
        17: "SORT_METHOD_COUNTRY",
        18: "SORT_METHOD_VIDEO_YEAR",
        19: "SORT_METHOD_VIDEO_RATING",
        20: "SORT_METHOD_VIDEO_USER_RATING",
        21: "SORT_METHOD_DATEADDED",
        22: "SORT_METHOD_PROGRAM_COUNT",
        23: "SORT_METHOD_PLAYLIST_ORDER",
        24: "SORT_METHOD_EPISODE",
        25: "SORT_METHOD_VIDEO_TITLE",
        26: "SORT_METHOD_VIDEO_SORT_TITLE",
        27: "SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE",
        28: "SORT_METHOD_PRODUCTIONCODE",
        29: "SORT_METHOD_SONG_RATING",
        30: "SORT_METHOD_SONG_USER_RATING",
        31: "SORT_METHOD_MPAA_RATING",
        32: "SORT_METHOD_VIDEO_RUNTIME",
        33: "SORT_METHOD_STUDIO",
        34: "SORT_METHOD_STUDIO_IGNORE_THE",
        35: "SORT_METHOD_FULLPATH",
        36: "SORT_METHOD_LABEL_IGNORE_FOLDERS",
        37: "SORT_METHOD_LASTPLAYED",
        38: "SORT_METHOD_PLAYCOUNT",
        39: "SORT_METHOD_LISTENERS",
        40: "SORT_METHOD_UNSORTED",
        41: "SORT_METHOD_CHANNEL",
        43: "SORT_METHOD_BITRATE",
        44: "SORT_METHOD_DATE_TAKEN",
    }

    def __init__(self, handle):
        self.handle = handle
        self.update_listing = False
        self.cache_to_disc = False
        self.succeeded = False
        self.content = "not-set"
        self.sort_methods = set()

        self.__items = []

    def add_item(self, list_item, url, is_folder):
        """ Add a new ListItem object with an url

        :param ListItem list_item:  The ListItem
        :param str url:             The url for the item
        :param bool is_folder:      Indiction whether it is a folder or not

        """

        self.__items.append((list_item, url, is_folder))

    @property
    def count(self):
        """ The number for items in the handle.

        :return: the number for items
        :rtype: int

        """

        return len(self.__items)

    @property
    def items(self):
        """ The current items for the handle

        :return: iterator for the items
        :rtype: list_iterator

        """

        return iter(self.__items)

    def print_handle(self):
        """ Prints the content for this handle """

        KodiStub.print_heading("Listing for handle {}".format(self.handle))

        for listitem, url, is_folder in self.items:
            if is_folder:
                print("*F: %s [%s]" % (KodiStub.replace_colors(str(listitem)), url))
            else:
                print("*V: %s [%s]" % (KodiStub.replace_colors(str(listitem)), url))

        for sort_method in self.sort_methods:
            KodiStub.print_line("{}>{} Added sortmethod: {:02d} - {}".format(
                Colors.Yellow, Colors.EndColor, sort_method,
                HandleInfo.__sort_method_names.get(sort_method, "<unknown")))

        KodiStub.print_heading(
            "End of Folder (items={},success={},content={},sort={},cache={},update={})".format(
                self.count,
                self.succeeded,
                self.content,
                "+".join([str(i) for i in self.sort_methods]),
                self.cache_to_disc,
                self.update_listing
            ), align_right=True)


class PluginHandler(object):
    __handles = dict()

    @staticmethod
    def get_handle_info(handle):
        """ Retrieves the info object for the Handle

        :param int handle:  The handle ID from Kodi

        :rtype: HandleInfo
        :returns: A complete HandleInfo object

        """

        if handle not in PluginHandler.__handles:
            PluginHandler.__handles[handle] = HandleInfo(handle)

        return PluginHandler.__handles[handle]

    @staticmethod
    def close_handle(handle):
        """ Closes a handle and removes it from the pool

        :param int handle:      The handle to close

        """

        if handle not in PluginHandler.__handles:
            return

        del PluginHandler.__handles[handle]



# SPDX-License-Identifier: GPL-3.0

import io
import os
from shutil import copyfile

from sakee import addoninfo
from sakee.colors import Colors
from sakee.stub import KodiStub


class File(object):  # NOSONAR
    def __init__(self, path, flags='r'):
        """ File class.

        :param str path:    The file or directory to open
        :param str flags:   The flags used to open the file
        """
        if flags not in ['r', 'w']:
            raise ValueError("flags should be 'r' or 'w'")

        self._file = io.open(path, flags + 'b')

    def close(self):
        """ Close the file. """
        self._file.close()

    # noinspection PyShadowingBuiltins
    def read(self, bytes=-1):
        """ Read from the file.

        :param int bytes:       How many bytes to read

        :return: The read bytes
        :rtype: str
        """
        # noinspection PyUnresolvedReferences
        return self._file.read(bytes).decode('utf-8')

    # noinspection PyPep8Naming
    def readBytes(self, numbytes=-1):  # NOSONAR
        """ Read from the file.

        :param int numbytes:    How many bytes to read

        :return: The read bytes
        :rtype: str
        """
        return self._file.read(numbytes)

    # noinspection PyPep8Naming
    def seek(self, seekBytes, iWhence=0):  # NOSONAR
        """ Seek to position in file.

        :param int seekBytes:   Position in the file
        :param int iWhence:     Where in a file to seek from (0=beginning, 1=current, 2=end position)

        :return: The current position in the file
        :rtype: int
        """
        return self._file.seek(seekBytes, iWhence)

    def size(self):
        """ Get the file size.

        :return: The file size
        :rtype: int
        """
        self._file.flush()
        return os.fstat(self._file.fileno()).st_size

    def tell(self):
        """ Get the current position in the file.

        :return: The current position in the file
        :rtype: int
        """
        return self._file.tell()

    def write(self, buffer):
        """ Write to the file.

        :param str|byte buffer:      Data to write to the file

        :return: True if successful
        :rtype bool
        """
        if isinstance(buffer, bytes):
            # noinspection PyTypeChecker
            bytes_written = self._file.write(buffer)
        else:
            # noinspection PyTypeChecker
            bytes_written = self._file.write(buffer.encode())
        return bytes_written == len(buffer)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()


class Stat(object):  # NOSONAR
    def __init__(self, path):
        """ Stat class.

        :param str path:        The file or directory to stat
        """
        self._stat = os.stat(path)

    def st_atime(self):
        """ Returns the st_atime attribute.

        :rtype: float
        :return: The st_atime attribute
        """
        return self._stat.st_atime

    def st_ctime(self):
        """ Returns the st_ctime attribute.

        :rtype: float
        :return: The st_ctime attribute
        """
        return self._stat.st_ctime

    def st_dev(self):
        """ Returns the st_dev attribute.

        :rtype: int
        :return: The st_dev attribute
        """
        return self._stat.st_dev

    def st_gid(self):
        """ Returns the st_gid attribute.

        :rtype: int
        :return: The st_gid attribute
        """
        return self._stat.st_gid

    def st_ino(self):
        """ Returns the st_ino attribute.

        :rtype: int
        :return: The st_ino attribute
        """
        return self._stat.st_ino

    def st_mode(self):
        """ Returns the st_mode attribute.

        :rtype: int
        :return: The st_mode attribute
        """
        return self._stat.st_mode

    def st_mtime(self):
        """ Returns the st_mtime attribute.

        :rtype: float
        :return: The st_mtime attribute
        """
        return self._stat.st_mtime

    def st_nlink(self):
        """ Returns the st_nlink attribute.

        :rtype: int
        :return: The st_nlink attribute
        """
        return self._stat.st_nlink

    def st_size(self):
        """ Returns the st_size attribute.

        :rtype: int
        :return: The st_size attribute
        """
        return self._stat.st_size

    def st_uid(self):
        """ Returns the st_uid attribute.

        :rtype: int
        :return: The st_uid attribute
        """
        return self._stat.st_uid


def copy(source, destination):
    """ Copy file to destination, returns true/false.

    :param str source:          The file to copy
    :param str destination:     The destination

    :return: True if successful
    :rtype bool
    """
    return copyfile(source, destination) == destination


def delete(file):
    """ Delete a file.

    :param str file:        File to delete

    :return: True if successful
    :rtype bool
    """
    try:
        os.remove(file)
        return True
    except OSError:
        return False


def exists(path):
    """ Check for a file or folder existence.

    :param str path:            File or folder (folder must end with slash or backslash)

    :return: True if the file exists
    :rtype bool
    """
    return os.path.exists(path)


def listdir(path):
    """ Lists content of a folder.

    :param str path:            Folder to get list from

    :return: Directory content list (directories, files)
    :rtype (list[str], list[str])
    """
    files = []
    dirs = []
    if not exists(path):
        return dirs, files
    for filename in os.listdir(path):
        fullname = os.path.join(path, filename)
        if os.path.isfile(fullname):
            files.append(filename)
        if os.path.isdir(fullname):
            dirs.append(filename)
    return dirs, files


# noinspection PyPep8Naming
def makeLegalFilename(path):  # NOSONAR
    """ Returns a legal filename or path as a string.

    :param str path:            Filename or path to make legal.

    :return: Legal filename or path as a string
    :rtype str
    """
    return os.path.normpath(path)


def mkdir(path):
    """ Create a directory.

    :param str path:            Directory to create

    :return: True if successful
    :rtype bool
    """
    return os.mkdir(path)


def mkdirs(path):
    """ Create a directory and all the directories along the path.

    :param str path:            Directory to create

    :return: True if successful
    :rtype bool
    """
    if os.path.exists(path):
        return True

    try:
        os.makedirs(path)
        return os.path.exists(path)
    except OSError:
        return False


def rmdir(path):
    """ Remove a directory.

    :param str path:            Directory to remove

    :return: True if successful
    :rtype bool
    """
    return os.rmdir(path)


# noinspection PyPep8Naming
def validatePath(path):  # NOSONAR
    """ Returns the validated path.

    :param str path:            Path to format

    :return: The validated path
    :rtype str
    """
    return os.path.normpath(path)


# noinspection PyPep8Naming
def translatePath(path):  # NOSONAR
    """ Returns the translated path.

    :param str path:    Path to format

    :return: Translated path
    :rtype: str

    See http://kodi.wiki/view/Special_protocol

    E.g:
        special://home/ is mapped to: kodi/
        special://profile/ is mapped to: kodi/userdata

    Or in portable:
        special://home/ is mapped to: kodi/portable_data/
        special://profile/ is mapped to: kodi/portable_data/userdata

    """

    def get_return_path(base_path, name, *segments):
        if not base_path:
            raise ValueError("Missing __kodi_{}_path data".format(name))
        new_path = os.path.join(base_path, *[i.replace("/", os.sep) for i in segments if i and i != ''])

        if not os.path.exists(new_path):
            raise ValueError("Invalid path specified: {}".format(path, ))

        return new_path

    if path.startswith("special://profile/"):
        return_path = get_return_path(__add_on_info.kodi_profile_path,
                                      "profile",
                                      path.replace("special://profile/", ""))

    elif path.startswith("special://home/"):
        return_path = get_return_path(__add_on_info.kodi_home_path,
                                      "home",
                                      path.replace("special://home/", ""))

    elif path.startswith("special://xbmcbin/"):
        return_path = get_return_path(__add_on_info.kodi_home_path,
                                      "home",
                                      "system",
                                      path.replace("special://xbmcbin/", ""))

    elif os.path.isabs(path):
        return path

    else:
        raise ValueError("Invalid special path: %s" % (path,))

    actual_path = os.path.abspath(return_path)
    KodiStub.print_line("Mapped '{0}' -> '{1}'".format(path, actual_path), color=Colors.Blue)
    return actual_path


__add_on_info = addoninfo.get_add_on_info_from_calling_script()



# SPDX-License-Identifier: GPL-3.0


class Colors:
    def __init__(self):
        pass

    EndColor = '\033[0m'
    # Some styles
    # BOLD = '\033[1m'
    # UNDERLINE = '\033[4m'

    White = '\033[30m'
    Red = '\033[31m'
    Green = '\033[32m'
    Orange = '\033[33m'
    Blue = '\033[34m'
    Purple = '\033[35m'
    Cyan = '\033[36m'
    LightGrey = '\033[37m'
    DarkGrey = '\033[90m'
    LightRed = '\033[91m'
    LightGreen = '\033[92m'
    Yellow = '\033[93m'
    LightBlue = '\033[94m'
    Pink = '\033[95m'
    LightCyan = '\033[96m'


if __name__ == '__main__':
    for color in dir(Colors):
        if color.startswith("__"):
            continue
        exec("print(Colors.%s + color + Colors.EndColor)" % (color, ))



# coding=utf-8  # NOSONAR
# SPDX-License-Identifier: GPL-3.0
import io
import os.path
from setuptools import setup

project_dir = os.path.dirname(os.path.abspath(__file__))


setup(
    name='sakee',
    version="0.0.9",
    url='https://github.com/retrospect-addon/kodi.emulator.ascii',
    author='Bas Rieter',
    description='SAKÉ can help you to debug and develop Kodi Python add-ons',
    long_description=io.open(os.path.join(project_dir, 'README.md'), encoding='utf-8').read(),
    long_description_content_type="text/markdown",
    keywords='Kodi, emulator, ascii, xbmc, xbmcgui, xbmcplugin, xbmcaddon',
    license='GPL-3.0',
    py_modules=["xbmc", "xbmcgui", "xbmcaddon", "xbmcgui", "inputstreamhelper", "xbmcplugin", "xbmcvfs"],
    packages=["sakee"],
    project_urls={
        'Documentation': 'https://github.com/retrospect-addon/kodi.emulator.ascii/blob/master/README.md',
        'Source': 'https://github.com/retrospect-addon/kodi.emulator.ascii/',
        'Tracker': 'https://github.com/retrospect-addon/kodi.emulator.ascii/issues',
    },
    zip_safe=False,
    python_requires='>=2.7'
)



# SPDX-License-Identifier: GPL-3.0

from sakee.colors import Colors
from sakee.internalplayer import KodiInteralPlayer
from sakee.pluginhandler import PluginHandler
from sakee.stub import KodiStub
from xbmcgui import ListItem

SORT_METHOD_ALBUM = 14
SORT_METHOD_ALBUM_IGNORE_THE = 15
SORT_METHOD_ARTIST = 11
SORT_METHOD_ARTIST_IGNORE_THE = 13
SORT_METHOD_BITRATE = 43
SORT_METHOD_CHANNEL = 41
SORT_METHOD_COUNTRY = 17
SORT_METHOD_DATE = 3
SORT_METHOD_DATEADDED = 21
SORT_METHOD_DATE_TAKEN = 44
SORT_METHOD_DRIVE_TYPE = 6
SORT_METHOD_DURATION = 8
SORT_METHOD_EPISODE = 24
SORT_METHOD_FILE = 5
SORT_METHOD_FULLPATH = 35
SORT_METHOD_GENRE = 16
SORT_METHOD_LABEL = 1
SORT_METHOD_LABEL_IGNORE_FOLDERS = 36
SORT_METHOD_LABEL_IGNORE_THE = 2
SORT_METHOD_LASTPLAYED = 37
SORT_METHOD_LISTENERS = 39
SORT_METHOD_MPAA_RATING = 31
SORT_METHOD_NONE = 0
SORT_METHOD_PLAYCOUNT = 38
SORT_METHOD_PLAYLIST_ORDER = 23
SORT_METHOD_PRODUCTIONCODE = 28
SORT_METHOD_PROGRAM_COUNT = 22
SORT_METHOD_SIZE = 4
SORT_METHOD_SONG_RATING = 29
SORT_METHOD_SONG_USER_RATING = 30
SORT_METHOD_STUDIO = 33
SORT_METHOD_STUDIO_IGNORE_THE = 34
SORT_METHOD_TITLE = 9
SORT_METHOD_TITLE_IGNORE_THE = 10
SORT_METHOD_TRACKNUM = 7
SORT_METHOD_UNSORTED = 40
SORT_METHOD_VIDEO_RATING = 19
SORT_METHOD_VIDEO_RUNTIME = 32
SORT_METHOD_VIDEO_SORT_TITLE = 26
SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE = 27
SORT_METHOD_VIDEO_TITLE = 25
SORT_METHOD_VIDEO_USER_RATING = 20
SORT_METHOD_VIDEO_YEAR = 18


# noinspection PyUnusedLocal,PyPep8Naming
def addDirectoryItem(handle, url, listitem, isFolder=False, totalItems=0):  # NOSONAR
    """ Callback function to pass directory contents back to Kodi.

    :param int handle:              Handle the plugin was started with.
    :param str url:                 Url of the entry. would be plugin:// for another virtual directory
    :param ListItem listitem:       Item to add.
    :param isFolder:                True=folder / False=not a folder(default).
    :param int totalItems:          Total number of items that will be passed.(used for progressbar)

    :return: Returns a bool for successful completion.
    :rtype: bool

    NOTE: You can use the above as keywords for arguments and skip certain optional arguments.
          Once you use a keyword, all following arguments require the keyword.

    """

    handle_info = PluginHandler.get_handle_info(handle)
    handle_info.add_item(listitem, url, isFolder)
    return True


# noinspection PyUnusedLocal,PyPep8Naming
def addDirectoryItems(handle, items, totalItems=0):  # NOSONAR
    """ Callback function to pass directory contents back to Kodi as a list.

    :param int handle:                          Handle the plugin was started with.
    :param list[(str, ListItem, bool)] items:   List of (url, listitem[, isFolder]) as a tuple to add.
    :param int totalItems:                      Total number of items that will be
                                                 passed. (used for progressbar)

    :return: Returns a bool for successful completion.
    :rtype: bool

    NOTE: Large lists benefit over using the standard addDirectoryItem(). You may call this more
          than once to add items in chunks.

    """

    result = True
    for item in items:
        url, item, is_folder = item
        result = result and addDirectoryItem(handle, url, item, is_folder, totalItems)

    return result


# noinspection PyPep8Naming,PyUnusedLocal
def endOfDirectory(handle, succeeded=True, updateListing=False, cacheToDisc=True):  # NOSONAR
    """ Callback function to tell Kodi that the end of the directory listing in a virtual
    Python folder module is reached.

    :param int handle:      Handle the plugin was started with.
    :param bool succeeded:  True=script completed successfully(Default)/False=Script did not.
    :param updateListing:   True=this folder should update the current listing/False=Folder
                             is a subfolder(Default).
    :param cacheToDisc:     True=Folder will cache if extended time(default)/False=this folder
                             will never cache to disc.

    """

    handle_info = PluginHandler.get_handle_info(handle)
    handle_info.update_listing = updateListing
    handle_info.cache_to_disc = cacheToDisc
    handle_info.succeeded = succeeded
    handle_info.print_handle()
    PluginHandler.close_handle(handle)

    # In case there was something playing force to stop it now.
    KodiInteralPlayer.instance().stop_playback(force=True)


# noinspection PyPep8Naming,PyUnusedLocal
def addSortMethod(handle, sortMethod, label2Mask="%D"):  # NOSONAR
    """ Adds a sorting method for the media list.

    :param int handle:      Handle the plugin was started with.
    :param int sortMethod:  See available sort methods at the bottom (or see SortFileItem.h).
    :param str label2Mask:  The label mask to use for the second label. Defaults to %D
    :return:

    softMethod can have: SORT_METHOD_NONE, SORT_METHOD_LABEL, SORT_METHOD_LABEL_IGNORE_THE,
    SORT_METHOD_DATE, SORT_METHOD_SIZE, SORT_METHOD_FILE, SORT_METHOD_DRIVE_TYPE,
    SORT_METHOD_TRACKNUM, SORT_METHOD_DURATION, SORT_METHOD_TITLE, SORT_METHOD_TITLE_IGNORE_THE,
    SORT_METHOD_ARTIST, SORT_METHOD_ARTIST_IGNORE_THE, SORT_METHOD_ALBUM,
    SORT_METHOD_ALBUM_IGNORE_THE, SORT_METHOD_GENRE, xbmcplugin.SORT_SORT_METHOD_VIDEO_YEAR,
    SORT_METHOD_YEAR, SORT_METHOD_VIDEO_RATING, SORT_METHOD_PROGRAM_COUNT,
    SORT_METHOD_PLAYLIST_ORDER, SORT_METHOD_EPISODE, SORT_METHOD_VIDEO_TITLE,
    SORT_METHOD_VIDEO_SORT_TITLE, SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE,
    SORT_METHOD_PRODUCTIONCODE, SORT_METHOD_SONG_RATING, SORT_METHOD_MPAA_RATING,
    SORT_METHOD_VIDEO_RUNTIME, SORT_METHOD_STUDIO, SORT_METHOD_STUDIO_IGNORE_THE,
    SORT_METHOD_UNSORTED, SORT_METHOD_BITRATE, SORT_METHOD_LISTENERS, SORT_METHOD_COUNTRY,
    SORT_METHOD_DATEADDED, SORT_METHOD_FULLPATH, SORT_METHOD_LABEL_IGNORE_FOLDERS,
    SORT_METHOD_LASTPLAYED, SORT_METHOD_PLAYCOUNT, SORT_METHOD_CHANNEL, SORT_METHOD_DATE_TAKEN,
    SORT_METHOD_VIDEO_USER_RATING, SORT_METHOD_SONG_USER_RATING

    label2Mask applies to: SORT_METHOD_NONE, SORT_METHOD_UNSORTED, SORT_METHOD_VIDEO_TITLE,
        SORT_METHOD_TRACKNUM, SORT_METHOD_FILE, SORT_METHOD_TITLE, SORT_METHOD_TITLE_IGNORE_THE,
        SORT_METHOD_LABEL, SORT_METHOD_LABEL_IGNORE_THE, SORT_METHOD_VIDEO_SORT_TITLE,
        SORT_METHOD_FULLPATH, SORT_METHOD_VIDEO_SORT_TITLE_IGNORE_THE,
        SORT_METHOD_LABEL_IGNORE_FOLDERS, SORT_METHOD_CHANNEL

    """

    handle_info = PluginHandler.get_handle_info(handle)
    handle_info.sort_methods.add(sortMethod)


# noinspection PyPep8Naming,PyUnusedLocal
def setContent(handle, content):  # NOSONAR
    """ Sets the plugins content.

    :param int handle:  Handle the plugin was started with.
    :param str content: Content type (eg. movies)


    Available content strings: files, songs, artists, albums, movies, tvshows, episodes,
        musicvideos, videos, images, games,

    Remarks: Use videos for all videos which do not apply to the more specific mentioned ones like
        "movies", "episodes" etc. A good example is youtube.

    """

    handle_info = PluginHandler.get_handle_info(handle)
    handle_info.content = content


# noinspection PyPep8Naming,PyUnusedLocal
def setResolvedUrl(handle, succeeded, listitem):  # NOSONAR
    """ Callback function to tell Kodi that the file plugin has been resolved to a url

    :param int handle:          Handle the plugin was started with.
    :param bool succeeded:      True=script completed successfully/False=Script did not.
    :param ListItem listitem:   Item the file plugin resolved to for playback.

    """

    if succeeded:
        KodiStub.print_line("Item resolved to: {}".format(listitem), color=Colors.Blue)
        KodiInteralPlayer.instance().play_resolved_item(listitem.getPath(), listitem)
    else:
        KodiStub.print_line("Item failed to resolve: {}".format(listitem), color=Colors.Red)


# noinspection PyPep8Naming,PyUnusedLocal
def setPluginCategory(handle, category):  # NOSONAR
    """ Sets the plugins name for skins to display.

    :param int handle:      Handle the plugin was started with.
    :param str category:    Plugins sub category.

    """

    KodiStub.print_line("> {}".format(category), color=Colors.Blue)
